#include "AI8051U.h"

#define MAiN_Fosc 40000000L // 一个指令周期=一个机器周期=1/40M=25ns;
// 延时1us的时间为1000ns/25ns=40；
//  1/Fosc=x ns
//  1000ns/(1/Fosc)ns= x 个时钟周期
//  实际计算时：1000e-9*Fosc=x 个时钟周期
//  while(i--)四个时钟周期，(x-4)=NOP(val);延时1us所需要的时间
typedef unsigned char u8;
typedef unsigned int u16;
u8 LED1[3]; // 红色（R=255, G=0, B=0）
u8 LED2[3]; // 绿色（R=0, G=255, B=0）
u8 color_table[5][3] = {
    {0xFF, 0x00, 0x00}, // 红色
    {0x00, 0xFF, 0x00}, // 绿色
    {0x00, 0x00, 0xFF}, // 蓝色
    {0xFF, 0xFF, 0x00}, // 黄色
    {0xFF, 0x00, 0xFF}  // 紫色
};
#define LED_PIN P15
extern int _nop_();
//该延时函数仅在40Mhz下是准确的，其他频率需要自行计算NOPx()的值
void delay_us(u16 us)
{
    do
    {
        NOP(36);
    } while (us--); // us--需要先对us进行减一次，然后赋值给us,然后判断一次，一共需要4个指令周期，NOP36是36个指令周期，40*25=1000ns=1us
    // MOV R6,R7 //1个指令周期
    // DEC R7,#0X01//1个指令周期
    // CJNE R6,#0X00,C:0X0005//如果发生跳转，就是3个指令周期，正常情况为2个指令周期。
    //方案2 
    //NOP40();
}
void delay_ms(u16 ms) // unsigned int
{
    do
    {
        delay_us(1000); // 减少对函数的调用，增加性能
    } while (--ms);
}
void send_byte(unsigned char byte)
{
    unsigned char i;
    for (i = 0; i < 8; i++)
    {
        if (byte & 0x80)
        {                // 如果最高位为 1
            LED_PIN = 1; // 发1 低电平为0.3us
            delay_us(1); //
            LED_PIN = 0;
            NOP13();
        }
        else
        {                // 如果最高位为 0
            LED_PIN = 1; // 发0 高电平为0.3us
            NOP13();     //
            LED_PIN = 0; //
            delay_us(1);
        }
        byte <<= 1; // 左移 1 位，准备发送下一位
    }
}

// 颜色变化逻辑
void change_color()
{
    // 增加变化步长，使颜色变化更明显
    LED1[0] = (LED1[0] + 20) % 256; // 红色通道（R）的增量
    LED1[1] = (LED1[1] + 40) % 256; // 绿色通道（G）的增量
    LED1[2] = (LED1[2] + 60) % 256; // 蓝色通道（B）的增量

    LED2[0] = (LED2[0] + 30) % 256; // 红色通道（R）的增量
    LED2[1] = (LED2[1] + 50) % 256; // 绿色通道（G）的增量
    LED2[2] = (LED2[2] + 70) % 256; // 蓝色通道（B）的增量
}

void switch_to_next_color(u8 *LED, u8 index)
{
    unsigned char i;
    LED[0] = color_table[index][0]; // G
    LED[1] = color_table[index][1]; // R
    LED[2] = color_table[index][2]; // B
    
    for (i = 0; i < 3; i++)
    {
        send_byte(LED[i]); // 发送每个颜色字节（G, R, B）
    }
}
// 发送一个 LED 的数据（24 位）
void send_LED_data1()
{
    unsigned char i;
    for (i = 0; i < 3; i++)
    {
        send_byte(LED1[i]); // 发送 3 个字节
    }
}
void send_LED_data2()
{
    unsigned char i;
    for (i = 0; i < 3; i++)
    {
        send_byte(LED2[i]); // 发送 3 个字节
    }
}
void ws2812b_reset()
{
    LED_PIN = 0;
    delay_us(300); // 拉低时间大于 50 微秒即可
}
// 主程序
void main()
{
    

    u8 index1 = 0; // LED1 的颜色索引
    u8 index2 = 2; // LED2 的颜色索引
    P1M0 |= 0x20; P1M1 &= ~0x20; 
    while (1)
    {
        // ws2812b_reset();
        // send_LED_data1(); // 发送第一个 LED 的数据

        // send_LED_data2(); // 发送第二个 LED 的数据
        // change_color();
        ws2812b_reset();
        switch_to_next_color(LED1, index1);
        switch_to_next_color(LED2, index2);

        index1 = (index1 + 1) % 5; // 颜色表循环
        index2 = (index2 + 1) % 5; // 颜色表循环
        // 延时 5 毫秒，避免过快发送
        delay_ms(500);
    }
}

// void send_byte(unsigned char byte)
// {
//     unsigned char i;
//     for (i = 0; i < 8; i++)
//     {
//         if (byte & 0x80)
//         {                // 如果最高位为 1
//             LED_PIN = 1; //发1 低电平为0.3us
//             delay_us(1); //
//             LED_PIN = 0;
//             NOP13();
//         }
//         else
//         {                // 如果最高位为 0
//             LED_PIN = 1; // 发0 高电平为0.3us
//             NOP13(); //
//             LED_PIN = 0; //
//             delay_us(1);
//         }
//         byte <<= 1; // 左移 1 位，准备发送下一位
//     }
// }

// // WS2812B 协议：发送一个 LED 的数据（24 位 = 3 字节）
// void send_LED_data(unsigned char r, unsigned char g, unsigned char b)
// {
//     send_byte(g); // 发送绿色分量
//     send_byte(r); // 发送红色分量
//     send_byte(b); // 发送蓝色分量
// }

// // WS2812B 协议：复位信号（拉低引脚至少 50 微秒）
// void ws2812b_reset()
// {
//     LED_PIN = 0;
//     delay_us(300); // 拉低时间大于 50 微秒即可
// }

// // 主函数：驱动两个 WS2812B 灯，并实现颜色变换
// void main()
// {
//     unsigned char r1 = 255, g1 = 0, b1 = 0; // 第一个灯初始为红色
//     unsigned char r2 = 0, g2 = 255, b2 = 0; // 第二个灯初始为绿色
//     P1M1 = 0;
//     P1M0 = 1;
//     while (1)
//     {
//         ws2812b_reset();             // 发送复位信号
//         send_LED_data(r1, g1, b1);   // 发送第一个 LED 的颜色数据
//         send_LED_data(r2, g2, b2);   // 发送第二个 LED 的颜色数据

//         // 修改颜色值，循环变换颜色（颜色渐变）
//         r1 = (r1 + 10) % 256;
//         g1 = (g1 + 15) % 256;
//         b1 = (b1 + 20) % 256;

//         r2 = (r2 + 20) % 256;
//         g2 = (g2 + 10) % 256;
//         b2 = (b2 + 15) % 256;

//         delay_ms(1000); // 延时一段时间（50ms），便于观察颜色变化
//     }
// }

// void WS2812B_SendReset(void) {

//     WS2812B_IN = 0;  // 拉低80us
//     delay_us(300);
// }

// // 发送1码
// void WS2812B_SendOne(void) {
//     WS2812B_IN = 1;     // 拉高延时1us
//     delay_us(1);
//     WS2812B_IN = 0;     // 拉低延时0.3us
//     NOP12();
// }

// // 发送0码
// void WS2812B_SendZero(void) {
//     WS2812B_IN = 1;     // 拉高延时0.3us
//     NOP12();
//     WS2812B_IN = 0;     // 拉低延时1us
//     delay_us(1);
// }
// // 纯红光
// void WS2812B_SendRed(void) {
//     unsigned char i;

//     for (i = 0; i < 8; ++i)
//         WS2812B_SendZero();
//     for (i = 0; i < 8; ++i)
//         WS2812B_SendOne();
//     for (i = 0; i < 8; ++i)
//         WS2812B_SendZero();
// }
// // 纯绿光
// void WS2812B_SendGreen(void) {
//     unsigned char i;

//     for (i = 0; i < 8; ++i)
//         WS2812B_SendOne();
//     for (i = 0; i < 8; ++i)
//         WS2812B_SendZero();
//     for (i = 0; i < 8; ++i)
//         WS2812B_SendZero();
// }
// // 纯蓝光
// void WS2812B_SendBlue(void) {
//     unsigned char i;

//     for (i = 0; i < 3; ++i)
//         WS2812B_SendZero();
//     for (i = 0; i < 4; ++i)
//         WS2812B_SendZero();
//     for (i = 0; i < 8; ++i)
//         WS2812B_SendOne();
// }

// void main(void) {
//     P1M1 = 0;
//     P1M0 = 1;

//     while (1) {
//         WS2812B_SendRed();
//         WS2812B_SendBlue();
//         WS2812B_SendReset();
//         delay_ms(1000);
//         WS2812B_SendBlue();
//          WS2812B_SendGreen();
//          WS2812B_SendReset();
//         delay_ms(1000);
//         WS2812B_SendGreen();
//         WS2812B_SendRed();
//         WS2812B_SendReset();
//         delay_ms(1000);
//     }
// }
